import type { PrimitiveAtom, SetStateAction, WritableAtom } from 'jotai';
declare type Unsubscribe = () => void;
declare type AsyncStorage<Value> = {
    getItem: (key: string) => Promise<Value>;
    setItem: (key: string, newValue: Value) => Promise<void>;
    delayInit?: boolean;
    subscribe?: (key: string, callback: (value: Value) => void) => Unsubscribe;
};
declare type SyncStorage<Value> = {
    getItem: (key: string) => Value;
    setItem: (key: string, newValue: Value) => void;
    delayInit?: boolean;
    subscribe?: (key: string, callback: (value: Value) => void) => Unsubscribe;
};
declare type AsyncStringStorage = {
    getItem: (key: string) => Promise<string | null>;
    setItem: (key: string, newValue: string) => Promise<void>;
};
declare type SyncStringStorage = {
    getItem: (key: string) => string | null;
    setItem: (key: string, newValue: string) => void;
};
export declare function createJSONStorage<Value>(getStringStorage: () => AsyncStringStorage): AsyncStorage<Value>;
export declare function createJSONStorage<Value>(getStringStorage: () => SyncStringStorage): SyncStorage<Value>;
export declare function atomWithStorage<Value>(key: string, initialValue: Value, storage: AsyncStorage<Value> & {
    delayInit: true;
}): WritableAtom<Value, SetStateAction<Value>, Promise<void>>;
export declare function atomWithStorage<Value>(key: string, initialValue: Value, storage: AsyncStorage<Value>): WritableAtom<Promise<Value>, SetStateAction<Value>, Promise<void>>;
export declare function atomWithStorage<Value>(key: string, initialValue: Value, storage: SyncStorage<Value>): PrimitiveAtom<Value>;
export declare function atomWithStorage<Value>(key: string, initialValue: Value): PrimitiveAtom<Value>;
export declare function atomWithHash<Value>(key: string, initialValue: Value, options?: {
    serialize?: (val: Value) => string;
    deserialize?: (str: string) => Value;
    delayInit?: boolean;
    replaceState?: boolean;
    subscribe?: (callback: () => void) => () => void;
}): PrimitiveAtom<Value>;
export {};
